"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }// src/index.ts
var _path = require('path');
var _promises = require('fs/promises'); var _promises2 = _interopRequireDefault(_promises);
var _pluginutils = require('@rollup/pluginutils');
var _vite = require('vite');
var _tsmorph = require('ts-morph');

// src/utils.ts
function isNativeObj(value) {
  return Object.prototype.toString.call(value) === "[object Object]";
}
function mergeObjects(sourceObj, targetObj) {
  const loop = [
    {
      source: sourceObj,
      target: targetObj
    }
  ];
  while (loop.length) {
    const {source, target} = loop.pop();
    Object.keys(target).forEach((key) => {
      if (isNativeObj(target[key])) {
        if (!isNativeObj(source[key])) {
          source[key] = {};
        }
        loop.push({
          source: source[key],
          target: target[key]
        });
      } else if (Array.isArray(target[key])) {
        if (!Array.isArray(source[key])) {
          source[key] = [];
        }
        loop.push({
          source: source[key],
          target: target[key]
        });
      } else {
        source[key] = target[key];
      }
    });
  }
  return sourceObj;
}

// src/index.ts
var src_default = (options = {}) => {
  const {
    include = ["**/*.vue", "**/*.ts", "**/*.tsx"],
    exclude = "node_modules/**",
    root = process.cwd(),
    projectOptions = null,
    cleanVueFileName = false
  } = options;
  const filter = _pluginutils.createFilter.call(void 0, include, exclude);
  const sourceFiles = [];
  const project = new (0, _tsmorph.Project)(mergeObjects({
    compilerOptions: {
      declaration: true,
      emitDeclarationOnly: true,
      noEmitOnError: true
    },
    tsConfigFilePath: _path.resolve.call(void 0, root, "tsconfig.json"),
    skipAddingFilesFromTsConfig: true
  }, projectOptions != null ? projectOptions : {}));
  return {
    name: "vite:dts",
    apply: "build",
    enforce: "post",
    transform(code, id) {
      if (!code || !filter(id))
        return null;
      if (/\.vue(\?.*type=script.*)$/.test(id)) {
        const filePath = _path.resolve.call(void 0, root, _vite.normalizePath.call(void 0, id.split("?")[0]));
        sourceFiles.push(project.createSourceFile(filePath + (/lang.ts/.test(id) ? ".ts" : ".js"), code));
      } else if (/\.tsx?$/.test(id)) {
        const filePath = _path.resolve.call(void 0, root, _vite.normalizePath.call(void 0, id));
        sourceFiles.push(project.addSourceFileAtPath(filePath));
      }
    },
    async generateBundle(outputOptions) {
      const declarationDir = outputOptions.file ? _path.dirname.call(void 0, outputOptions.file) : outputOptions.dir;
      const diagnostics = project.getPreEmitDiagnostics();
      console.log(project.formatDiagnosticsWithColorAndContext(diagnostics));
      project.emitToMemory();
      for (const sourceFile of sourceFiles) {
        const emitOutput = sourceFile.getEmitOutput();
        for (const outputFile of emitOutput.getOutputFiles()) {
          const filePath = outputFile.getFilePath();
          const content = outputFile.getText();
          const targetPath = _path.resolve.call(void 0, declarationDir, _path.relative.call(void 0, root, cleanVueFileName ? filePath.replace(".vue.d.ts", ".d.ts") : filePath));
          await _promises2.default.mkdir(_path.dirname.call(void 0, targetPath), {recursive: true});
          await _promises2.default.writeFile(targetPath, cleanVueFileName ? content.replace(/['"](.+)\.vue['"]/g, '"$1"') : content, "utf8");
        }
      }
    }
  };
};


exports.default = src_default;
